@@ -59,14 +59,10 @@ char * strdup (const char *);
 #define TIMEOUT_LONG 604800
 /* don't fetch threads that have been "read" only once in this many seconds */
 #define TIMEOUT_SHORT 172800
-
-/* verbosity level, for fetch and later texpire */
-extern int verbose;
-
-/* handy malloc wrappers from util.c */
-
-char * critmalloc(size_t size, const char* message);
-char * critrealloc(char *a, size_t size, const char* message);
+/* time interval in seconds for completely rereading newsgroups list */
+#define READ_INTERVALL ( 90 * 24 * 3600 )
+/* Limit on the number of message bodies marked for download per group. */
+#define BODY_DOWNLOAD_LIMIT 2048
 
 /* converts a message-id to a file name, the return value points into
    a static array */
@@ -77,45 +73,45 @@ char * getaline(FILE *f);
 /* changes (and optionally creates) directory */
 void chdirgroup(const char *group);
 
-/* newsgroup management */
-
+/*
+ * newsgroup management
+ */
 struct newsgroup {
     struct newsgroup * left;
     struct newsgroup * right;
     int first;
     int last;
-    int server;
     int alive;
-    char * desc;
     char * name;
+    char * desc;
+    time_t age;
 };
 
-
-void insertgroup(const char * name, 
-			int first, int last, 
-			const char * desc, int server );
+void insertgroup(const char * name, int first, int last, const char * desc,
+	int date );
 struct newsgroup * findgroup(const char* name);
 void readactive( void );
 void writeactive( void );
+void fakeactive( void );
+
+/* newsgroup blocking */
+int findblocked(const char* name);
+void readblocked( void );
+extern struct newsgroup * blocked;
 
 extern struct newsgroup * active;
 
-/* supplementary servers */
-
-struct supplement {
-    struct supplement * next;
-    char name[1];
-};
-
-extern struct supplement * supplement;
-
 /* translation from message-id to article number, used in fetch and expire */
+
 void clearidtree( void );
 void insertmsgid( const char * msgid, int art );
-unsigned long int findmsgid( const char* msgid );
+int findmsgid( const char* msgid );
 
-/* xover stuff */
+/* -----------here starts the new stuff-----------------*/
 
+/*
+ * xover stuff -- for nntpd.c
+ */
 struct xoverinfo {
     char * text;
     int mallocd;
@@ -126,76 +122,90 @@ extern struct xoverinfo * xoverinfo;
 extern int xfirst;
 extern int xlast;
 
-/* return 1 if xover is a legal overview line, 0 else */
-int legalxoverline( const char * xover );
-/* set xoverinfo, return 0 if there's an error, nonzero else */
-int getxover( void );
-/* build a psuedo article for triggering fetches etc */
-FILE * buildpseudoart(const char * grp);
-/* An rfc type date */
-const char* rfctime(void);
-
-/* the strings in config.c */
+int getxover( void );	/* set xoverinfo, return 0 on error, nonzero else */
+int legalxoverline ( char * xover );	/* return 1 if xoverline is legal */
 
+/*
+ * the strings in config.c
+ */
 extern const char * spooldir;
 extern const char * libdir;
+extern const char * bindir;
 extern const char * version;
 extern const char * lockfile;
 
-extern char s[];
-
-/* my name, and my naming myself */
-extern char fqdn[];
-void whoami( void );
-
-
-extern FILE *nntpin;
-extern FILE *nntpout;
-
 /*
- * decode an NNTP reply number
- * reads a line from the server and returns an integer
- *
- * 498 is used to mean "protocol error", like smail
- *
- * the text returned is discarded
+ * global variables from config file. These are defined in configutil.c
  */
+struct expire_entry {
+    int xtime;
+    struct expire_entry *next;
+    char * group;
+};
 
-int nntpreply(void);
+struct serverlist {
+    struct serverlist * next;
+    char name[1] ;
+};
 
-/*
- * connect to upstream nntp server
- *
- * returns non-zero if a connection could be established
- */
-int nntpconnect( void );
-
-/* upstream news server */
-extern const char * upstream;
-/* max number of articles to read on one group in one go */
-extern int artlimit;
-/* articles not read or written since this time get deleted */
-extern time_t expire;
-/* max number of articles to read on one group the first time */
+extern char * upstream;	/* upstream news server */
+extern char * username;	/* authinfo user name */
+extern char * password;	/* authinfo password */
+extern time_t expire;	/* articles not touched since this time get deleted */
+extern struct expire_entry * expire_base;
+			/* expire for certain groups */
+extern int artlimit;	/* max # of articles to read per group in one go */
 extern int initiallimit;
-/* authinfo user name */
-extern const char * username;
-/* authinfo password */
-extern const char * password;
+			/* max # of articles to read at first time */
+extern int crosspostlimit;
+			/* crossposting limit, to reduce spam */
+extern int delaybody;	/* delay download of message body */
+extern int scarce_links;
+			/* create only necessary hardlinks */
+extern int maxage;	/* max age of articles */
+extern int maxlines;	/* max length of articles in lines */
+extern int nntpport;	/* NNTP port to connect to */
+extern struct serverlist * postserver;
+			/* list of servers to post through */
+extern struct serverlist * readserver;
+			/* list of servers to read from */
 
+/*
+ * other global variables
+ */
 extern char last_command[1025];
 extern char lineout[1025];
 
+/* defined in nntputil.c */
+extern FILE *nntpin;
+extern FILE *nntpout;
+
+extern char s[];
+extern char fqdn[];		/* my name, and my naming myself */
+
+extern int verbose;		/* verbosity level, for fetch and texpire */
+extern int debug;		/* debug level */
+
+/*
+ * misc prototypes
+ */
 void putaline( void );
 void retry( void );
 int authenticate( void );
 void readexpire( void );
-unsigned long lookup_expire(char *group);
+int lookup_expire(char *group);
 void free_expire( void );
+int readconfig( void );
+void whoami( void );
+char * critmalloc(size_t size, const char* message);
+char * critrealloc(char *a, size_t size, const char* message);
 
-/* read the config file */
-void readconfig( void );
+int nntpreply( void );		/* decode an NNTP reply number -- nntputil.c */
+int nntpconnect( const char * upstream );
+				/* connect to upstream server -- nntputil.c */
+
+FILE * buildpseudoart(const char * grp);
+			/* build a pseudo article for triggering fetches etc */
+const char* rfctime(void);	/* An rfc type date */
 
-/* expire stuff */
-struct newsgroup * getexpire( struct newsgroup * , char * );
 #endif
