@@ -1,11 +1,21 @@
-/* $Id: nntpd.c,v 1.27 1997/07/21 14:08:40 agulbra Exp $ */
+/*
+nntpd -- the NNTP server
 
-/* Written by Arnt Gulbrandsen <agulbra@troll.no> and copyright 1995
+Written by Arnt Gulbrandsen <agulbra@troll.no> and copyright 1995
 Troll Tech AS, Postboks 6133 Etterstad, 0602 Oslo, Norway, fax +47
 22646949.
+Modified by Cornelius Krasel <krasel@wpxx02.toxi.uni-wuerzburg.de>
+and Randolf Skerka <rskerka@metronet.de>.
+Copyright of the modifications 1997.
+Modified by Kent Robotti <robotti@erols.com>. Copyright of the
+modifications 1998.
+Modified by Markus Enzenberger <enz@cip.physik.uni-muenchen.de>.
+Copyright of the modifications 1998.
+Modified by Cornelius Krasel <krasel@wpxx02.toxi.uni-wuerzburg.de>.
+Copyright of the modifications 1998.
 
-Use, modification and distribution is allowed without limitation,
-warranty, or liability of any kind.  */
+See file COPYING for restrictions on the use of this software.
+*/
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -27,6 +37,8 @@ warranty, or liability of any kind.  */
 #include <sys/time.h>
 #include <unistd.h>
 
+const int verbose = 0;
+
 #include "leafnode.h"
 
 int write2(int fd, const char *msg);
@@ -39,14 +51,12 @@ void rereadactive(void);
 void parser(void);
 void error(const char *msg);
 void doarticle(const char *, int);
-void dostat(const char *);
 void dogroup(const char *);
 void dohelp(void);
 void domove(int);
 void domove(int);
 void dolist(const char *);
 void donewgroups(const char *);
-void donewnews(const char *);
 void dopost( void );
 void doxhdr(char *);
 void doxover(const char *);
@@ -63,12 +73,13 @@ time_t activetime;
 char fqdn[64];
 static int justaftergroup;
 
+int debug = 1;
 
 void rereadactive(void) {
     struct stat st;
 
-    strcpy(s, libdir);
-    strcat(s, "/groupinfo");
+    strcpy(s, spooldir);
+    strcat(s, "/leaf.node/groupinfo");
 
     if ( (!stat(s, &st) && (st.st_mtime > activetime)) ||
 	 (active==NULL) ) {
@@ -106,6 +117,7 @@ void parser(void) {
 	    continue; /* outside the spec, but let's do it anyway */
 
 	if ( n>999 ) {
+	    syslog( LOG_DEBUG, ">400 Dazed and confused" ); 
 	    printf("400 Dazed and confused\r\n");
 	    return;
 	}
@@ -115,8 +127,6 @@ void parser(void) {
 
 	arg = cmd+n;
 
-	syslog( LOG_DEBUG, "%s %s", cmd, arg);
-
 	while(cmd[n] && n<1000)
 	    n++;
 	n--;
@@ -143,17 +153,21 @@ void parser(void) {
 	} else if (!strcasecmp(cmd, "list")) {
 	    dolist(arg);
 	} else if (!strcasecmp(cmd, "mode")) {
+	    syslog( LOG_DEBUG, ">200 Modal interfaces are for losers" );
 	    printf("200 Modal interfaces are for losers\r\n");
 	} else if (!strcasecmp(cmd, "newgroups")) {
 	    donewgroups(arg);
 	} else if (!strcasecmp(cmd, "newnews")) {
-	    donewnews(arg);
+	    syslog( LOG_DEBUG,
+		    ">500 NEWNEWS is meaningless for this server\r\n" );
+	    printf( "500 NEWNEWS is meaningless for this server\r\n" );
 	} else if (!strcasecmp(cmd, "post")) {
 	    dopost();
 	} else if (!strcasecmp(cmd, "quit")) {
 	    printf("205 Always happy to serve!\r\n");
 	    return;
 	} else if (!strcasecmp(cmd, "slave")) {
+	    syslog( LOG_DEBUG, ">202 Cool - I always wanted a slave" );
 	    printf("202 Cool - I always wanted a slave\r\n");
 	} else if (!strcasecmp(cmd, "xhdr")) {
 	    doxhdr(arg);
@@ -163,8 +177,10 @@ void parser(void) {
 	    dolistgroup(arg);
 	} else if (!strcasecmp(cmd, "group")) {
 	    dogroup(arg);
-	} else
+	} else {
+	    syslog( LOG_DEBUG, ">500 Unknown command" );
 	    printf("500 Unknown command\r\n");
+	}
 	fflush(stdout);
     }
 }
@@ -177,9 +193,12 @@ FILE * fopenpseudoart(const char * arg, const int article_num) {
     char * c;
     struct newsgroup * g;    
 
-    if (article_num && (article_num < group->first)) {
+    syslog( LOG_DEBUG, "%s: first %d last %d artno %d",
+    		group->name, group->first, group->last, article_num );
+    if ( article_num && article_num == group->first &&
+         group->first >= group->last ) {
 	f = buildpseudoart(group->name);
-    } else if (!article_num) {
+    } else if ( !article_num ) {
 	if (!strncmp(arg, "<leafnode:placeholder:", 22)) {
 	    strncpy(msgidbuf, arg+22, 128);
 	    msgidbuf[127] = '\0';
@@ -198,7 +217,7 @@ FILE * fopenpseudoart(const char * arg, const int article_num) {
 
 /* open an article by number or message-id */
 FILE * fopenart(const char * arg) {
-    unsigned long int a;
+    int a;
     FILE *f;
     char *t;
 
@@ -218,7 +237,7 @@ FILE * fopenart(const char * arg) {
 	    f = fopenpseudoart(arg, a);
     } else if (group && artno) {
 	sprintf(s, "%d", artno);
-	printf( "fopenart: %s\n", s );
+	syslog( LOG_DEBUG, "fopenart: %s\n", s );
 	f = fopen(s, "r");
 	if (!f)
 	    f = fopenpseudoart(s, a);
@@ -230,6 +249,38 @@ FILE * fopenart(const char * arg) {
 }
     
 
+/*
+ * Mark an article for download by appending its number to the
+ * corresponding file in interesting.groups
+ */
+static int markdownload( long id )
+{
+    int i;
+    long n;
+    FILE *f;
+
+    snprintf( s, 1024,
+              "%s/interesting.groups/%s", spooldir, group->name );
+    if ( ( f = fopen( s, "r+" ) ) ) {
+        i = 0;
+        while ( fgets( s, 1024, f ) ) {
+            if ( sscanf( s, "%ld", &n ) == 1 && n == id )
+                return 0; /* already marked */
+            ++i;
+        }
+        if ( i < BODY_DOWNLOAD_LIMIT ) {
+            fprintf( f, "%ld\n", id );
+            syslog( LOG_DEBUG, "Marking %s %ld for download",
+                    group->name, id );
+        }
+        else {
+            syslog( LOG_ERR, "Too many bodies marked in %s",
+                    group->name );
+        }
+        fclose( f );
+    }
+    return 1;
+}
 
 /* display an article or somesuch */
 void doarticle(const char *arg, int what) {
@@ -239,16 +290,19 @@ void doarticle(const char *arg, int what) {
 
     if ( !group ) {
 	printf( "412 No newsgroup selected\r\n" );
+	syslog( LOG_DEBUG, ">412 No newsgroup selected\r\n" );
 	return;
     }
 
     f = fopenart(arg);
     if (!f) {
-	if ( strlen( arg ) )
+	if ( strlen( arg ) ) {
 	    printf("430 No such article: %s\r\n", arg );
-	else
-	    printf("430 No such article: %d\r\n", artno );
-	return;
+	    syslog( LOG_DEBUG, ">430 No such article: %s\r\n", arg );
+	} else {
+	    printf( "430 No such article: %d\r\n", artno );
+	    syslog( LOG_DEBUG, ">430 No such article: %d\r\n", artno );
+	} return;
     }
 
     yuck = strtol( arg, &p, 10 );
@@ -257,6 +311,9 @@ void doarticle(const char *arg, int what) {
 
     printf("%3d %ld Text follows, then a line consisting of a single '.'\r\n",
 	   223-what, yuck );
+    syslog( LOG_DEBUG, ">%3d %ld Text follows, "
+	       "then a line consisting of a single '.'\r\n",
+	       223-what, yuck );
 
     while (fgets(s, 1024, f) && *s && (*s!='\n')) {
 	if (what & 2) {
@@ -271,12 +328,28 @@ void doarticle(const char *arg, int what) {
     if (what==3)
 	printf("\r\n"); /* empty separator line */
 
-    while ((what & 1) && fgets(s, 1024, f) && *s) {
-	p = s;
-	if ( (p = strchr(p, '\n')) )
-	    *p = '\0';
-
-	printf("%s%s\r\n", *s=='.' ? "." : "", s);
+    if ( what & 1 ) {
+	if ( delaybody && *s != '\n' ) {
+	    if ( ! markdownload( yuck ) ) {
+		printf( "\r\n\r\n"
+			"\t[ Leafnode: ]\r\n"
+			"\t[ This message has already been "
+			"marked for download. ]\r\n" );
+	    } else {
+		printf( "\r\n\r\n"
+			"\t[ Leafnode: ]\r\n"
+			"\t[ Message %ld of %s ]\r\n"
+			"\t[ has been marked for download. ]\r\n",
+			yuck, group->name );
+	    }
+	} else {
+	    while ( fgets(s, 1024, f) && *s) {
+		p = s;
+		if ( (p = strchr(p, '\n')) )
+		    *p = '\0';
+		printf("%s%s\r\n", *s=='.' ? "." : "", s);
+	    }
+	}
     }
     if ( what )
 	printf(".\r\n");
@@ -310,20 +383,31 @@ void dogroup(const char *arg) {
     if (g) {
 	group = g;
 	chdirgroup( g->name );
-	if (g->last < g->first)
+	if (g->last < g->first) {
+	    syslog( LOG_DEBUG,
+	           ">211 %d %d %d %s group selected (pseudo article)\r\n",
+		   1, 1, 1, g->name);
 	    printf("211 %d %d %d %s group selected (pseudo article)\r\n", 
-		   1, g->last, g->last, g->name);
-	else
+		   1, 1, 1, g->name);
+	} else {
+	    syslog( LOG_DEBUG,
+	    	   ">211 %d %d %d %s group selected\r\n",
+		   g->last >= g->first ? g->last-g->first+1 : 0,
+		   g->first, g->last, g->name);
 	    printf("211 %d %d %d %s group selected\r\n", 
 		   g->last >= g->first ? g->last-g->first+1 : 0, 
 		   g->first, g->last, g->name);
+	}
 	artno = g->first;
 	fflush( stdout );
 	justaftergroup = TRUE;
+/* I don't think nntpd needs this -- CK
 	sprintf( s, "%s/interesting.groups/.%s", spooldir, group->name );
+*/
 	if ( (f=open( s, O_WRONLY|O_CREAT|O_TRUNC, 0644)) >= 0 )
 	    close(f);
     } else {
+    	syslog( LOG_DEBUG, ">411 No such group\r\n");
 	printf("411 No such group\r\n");
     }
 }
@@ -338,15 +422,26 @@ void domove( int by ) {
     if ( group ) {
 	if (artno) {
 	    artno += by;
-	    if ( artno > group->last )
+	    if ( artno > group->last ) {
 		artno = group->last;
-	    if ( artno < group->first )
+		printf("421 There is no next article\r\n" );
+		syslog( LOG_DEBUG, ">421 There is no next article" );
+	    }
+	    else if ( artno < group->first ) {
 		artno = group->first;
-	    printf( "223 Article number is now %d\r\n", artno);
+		printf("421 There is no previous article\r\n" );
+		syslog( LOG_DEBUG, ">421 There is no previous article" );
+	    }
+	    else {
+	        printf( "223 Article number is now %d\r\n", artno);
+		syslog( LOG_DEBUG, ">223 Article number is now %d", artno );
+	    }
 	} else {
 	    printf( "420 There is no current article\r\n" );
+	    syslog( LOG_DEBUG, ">420 There is no current article" );
 	}
     } else {
+	syslog( LOG_DEBUG, ">412 No newsgroup selected" );
 	printf( "412 No newsgroup selected\r\n" );
     }
 }
@@ -362,10 +457,11 @@ void list(struct newsgroup *g, int what) {
 	} else {
 	    /* If the group is empty, but has server arts, fudge a
                dummy article in... */
-	    if (g->last < g->first)
+	    if (g->last < g->first) {
 		printf( "%s %010d %010d y\r\n", g->name, g->last, g->last);
-	    else
+	    } else {
 		printf( "%s %010d %010d y\r\n", g->name, g->last, g->first);
+	    }
 	}
 	list (g->left, what);
     }
@@ -375,41 +471,87 @@ void list(struct newsgroup *g, int what) {
 void dolist(const char *arg) {
 
     rereadactive();
-    if (chdir(spooldir))
+    if (chdir(spooldir)) {
 	printf("503 News spool directory does not exist!\r\n");
-    else if (!active)
+	syslog( LOG_ERR, ">503 News spool directory does not exist!" );
+    } else if (!active) {
 	printf("503 Group information file does not exist!\r\n");
-    else {
+	syslog( LOG_ERR, ">503 Group information file does not exist!" );
+    } else {
 	if ( !*arg || !strcasecmp(arg, "active") ) {
 	    printf("215 Newsgroups in form \"group high low flags\".\r\n");
+	    syslog( LOG_DEBUG,
+		    ">215 Newsgroups in form \"group high low flags\"." );
 	    list ( active, 0 );
 	    printf(".\r\n");
 	} else if ( !strcasecmp(arg, "newsgroups") ) {
 	    printf("215 Descriptions in form \"group description\".\r\n");
+	    syslog( LOG_DEBUG,
+		   ">215 Descriptions in form \"group description\"." );
 	    list ( active, 1 );
 	    printf(".\r\n");
 	} else if ( !strcasecmp(arg, "extensions") ) {
 	    printf("501 No extensions available.\r\n" );
+	    syslog( LOG_DEBUG, ">501 No extensions available." );
+	} else if ( !strcasecmp(arg, "overview.fmt") ) {
+	    printf("215 information follows\r\n"
+		   "Subject:\r\n"
+		   "From:\r\n"
+		   "Date:\r\n"
+		   "Message-ID:\r\n"
+		   "References:\r\n"
+		   "Bytes:\r\n"
+		   "Lines:\r\n"
+		   "Xref:full\r\n"
+	           ".\r\n");
+	    syslog( LOG_DEBUG, ">215 information follows" );
 	} else if ( !strcasecmp(arg, "active.times") ) {
 	    printf("215 Placeholder - Leafnode will fetch groups on demand\r\n"
 		   "news.announce.newusers 42 tale@uunet.uu.net\r\n"
 		   "news.answers 42 tale@uunet.uu.net\r\n"
 		   ".\r\n");
+	    syslog( LOG_DEBUG,
+		    ">215 Placeholder - Leafnode will fetch groups on demand" );
 	} else {
 	    printf( "501 Syntax error\r\n" );
+	    syslog( LOG_DEBUG, ">501 Syntax error" );
 	}
     }
 }
 
-	
+static void helpdonewgroups( struct newsgroup *g, time_t age ) {
+    if ( g ) {
+	helpdonewgroups( g->right, age );
+	if ( g->age >= age )
+	    printf( "%s %d %d y\r\n", g->name, g->first, g->last );
+	helpdonewgroups( g->left, age );
+    }
+}
+
 void donewgroups(const char *arg) {
-    printf( "231 Command not supported\r\n.\r\n" );
-}
+    struct tm timearray;
+    time_t age;
+    char * l;
+    int a;
 
-void donewnews(const char *arg) {
-    printf( "500 NEWNEWS is meaningless for this server\r\n" );
-}
+    l = NULL;
+    a = strtol( arg, &l, 10 );
+    timearray.tm_year = a / 10000;
+    timearray.tm_mon  = (a % 10000 / 100) - 1 ;
+    timearray.tm_mday = a % 100 ;
+    while ( *l && isspace (*l) )
+	l++;
+    a = strtol( l, NULL, 10 );	/* we don't care about the rest of the line */
+    timearray.tm_hour = a / 10000;
+    timearray.tm_min  = a % 10000 / 100;
+    timearray.tm_sec  = a % 100;
+    age = mktime( &timearray ) ;
 
+    printf( "231 List of new newsgroups follows\r\n" );
+    syslog( LOG_DEBUG, ">231 List of new newsgroups follows" );
+    helpdonewgroups( active, age );
+    printf( ".\r\n" );
+}
 
 /* next bit is copied from INN 1.4 and modified ("broken") by agulbra
 
@@ -445,7 +587,7 @@ char * generateMessageID( void ) {
 	n >>= 5;
     }
     *p++ = '.';
-    n = fudge * 32768 + (unsigned long)getpid();
+    n = fudge * 32768 + (int)getpid();
     while( n ) {
 	*p++ = ALPHABET[(int)(n & 31)];
 	n >>= 5;
@@ -482,16 +624,20 @@ void dopost( void ) {
 	out = open( outname, O_WRONLY|O_EXCL|O_CREAT, 0444 );
 	if ( out < 0 ) {
 	    printf( "440 Unable to open spool file %s\r\n", outname );
+	    syslog( LOG_ERR, ">440 Unable to open spool file %s", outname );
 	    return;
 	}
     }
 
     printf( "340 Go ahead.\r\n" );
+    syslog( LOG_DEBUG, ">340 Go ahead.\r\n" );
     fflush( stdout );
 
     do {
 	int len;
+	debug = 0;
 	line = getaline( stdin );
+	debug = 1;
 	if ( !line ) {
 	    unlink( outname );
 	    exit( 0 );
@@ -564,7 +710,9 @@ void dopost( void ) {
     } while ( *line );
 
     do {
+	debug = 0;
 	line = getaline( stdin );
+	debug = 1;
 	if ( !line ) {
 	    unlink( outname );
 	    exit( 1 );
@@ -587,12 +735,13 @@ void dopost( void ) {
 
     if ( havefrom && havesubject && havenewsgroups && !err ) {
 	printf( "240 Article posted, now be patient\r\n" );
+	syslog( LOG_DEBUG, ">240 Article posted, now be patient\r\n" );
 	return;
     }
 
     unlink( outname );
     printf( "440 Formatting error, article not posted\r\n" );
-
+    syslog( LOG_DEBUG, ">440 Formatting error, article not posted\r\n" );
 }
 
 
@@ -604,7 +753,7 @@ void doxhdr(char *arg) {
 
     int n = 7;
     char * l;
-    unsigned long int a,b,c;
+    int a,b,c;
     FILE * f;
 
     l = arg;
@@ -620,6 +769,7 @@ void doxhdr(char *arg) {
 	f = fopenart(l);
 	if (!f) {
 	    printf("430 No such article\r\n");
+	    syslog( LOG_DEBUG, ">430 No such article" );
 	    return;
 	}
 	do {
@@ -631,6 +781,7 @@ void doxhdr(char *arg) {
 	    l++;
 	while( l && *l && isspace(*l) )
 	    l++;
+	syslog( LOG_DEBUG, "221 First line of %s header follows:", arg );
 	printf( "221 First line of %s header follows:\r\n%s\r\n.\r\n",
 		arg, l ? l : "" ); /* dubious - if not found, "" */
 	fclose( f );
@@ -648,7 +799,9 @@ void doxhdr(char *arg) {
 	    while (l && isspace(*l))
 		l++;
 	    if ( l && *l ) {
-		printf( "502 Usage: XHDR header first[-last] or XHDR header message-id\r\n" );
+		syslog( LOG_DEBUG,
+			">502 Usage: XHDR header first[-last] or XHDR header message-id\r\n" );
+		printf( "502 Usage: XHDR header first[-last] or XHDR header message-id" );
 		return;
 	    }
 	    if ( b==0 )
@@ -659,11 +812,14 @@ void doxhdr(char *arg) {
     }
 
     if (!group) {
+	syslog( LOG_DEBUG, ">412 Use the GROUP command first " );
 	printf( "412 Use the GROUP command first\r\n" );
 	return;
     }
 
     if ( !arg || !*arg ) {
+	syslog( LOG_DEBUG,
+		"502 Usage: XHDR header first[-last] or XHDR header message-id" );
 	printf( "502 Usage: XHDR header first[-last] or XHDR header message-id\r\n" );
 	return;
     }
@@ -685,7 +841,9 @@ void doxhdr(char *arg) {
 	b = group->last;
 
     if ( n >= 0 ) {
-	printf( "221 %s header (from overview) for postings %lu-%lu:\r\n",
+	syslog( LOG_DEBUG,
+		"221 %s header (from overview) for postings %d-%d:", h[n],a,b );
+	printf( "221 %s header (from overview) for postings %d-%d:\r\n",
 		h[n], a, b );
 
 	if ( xovergroup != group ) {
@@ -709,15 +867,17 @@ void doxhdr(char *arg) {
 		    if (p)
 			*p = '\0';
 		}
-		printf("%lu %s\r\n", c, l && *l ? s : "");
+		printf("%d %s\r\n", c, l && *l ? s : "");
 	    }
 	}
     } else {
-	printf( "221 %s header (from article files) for postings %lu-%lu:\r\n",
+	syslog( LOG_DEBUG,
+		">221 %s header (from overview) for postings %d-%d:",arg,a,b );
+	printf( "221 %s header (from article files) for postings %d-%d:\r\n",
 		arg, a, b );
 	n = strlen( arg );
 	for( c=a; c<=b; c++ ) {
-	    sprintf( s, "%lu", c );
+	    sprintf( s, "%d", c );
 	    f = fopen( s, "r");
 	    while( f ) {
 		l = getaline( f );
@@ -726,7 +886,7 @@ void doxhdr(char *arg) {
 		    f = 0;
 		} else if ( l[n] == ':' &&
 			    strncasecmp( arg, l, n ) == 0 ) {
-		    printf("%lu %s\r\n", c, l + n + 2 );
+		    printf("%d %s\r\n", c, l + n + 2 );
 		}
 	    }
 	}
@@ -738,11 +898,13 @@ void doxhdr(char *arg) {
 
 
 void doxover(const char *arg) {
+    FILE * f;
     char * l;
-    unsigned long int a,b, art;
+    int a,b, art;
 
     if (!group) {
 	printf( "412 Use the GROUP command first\r\n" );
+	syslog( LOG_DEBUG, ">412 Use the GROUP command first" );
 	return;
     }
 
@@ -759,12 +921,29 @@ void doxover(const char *arg) {
 	    l++;
 	if ( l && *l ) {
 	    printf( "502 Usage: XOVER first[-last]\r\n" );
+	    syslog( LOG_DEBUG, ">502 Usage: XOVER first[-last]" );
 	    return;
 	}
     }
 
     if ( xovergroup != group && !getxover() ) {
-	printf( "224 Overview information not available\r\n.\r\n" );
+        if ( ( f = fopen(".overview", "r" ) ) != NULL ) {
+	    syslog( LOG_DEBUG, "Overview file present but newsgroup unknown" );
+	}
+	printf( "224 Overview information (pseudo) for postings %d-%d:\r\n",
+		a, b );
+	syslog( LOG_DEBUG,
+		">224 Overview information (pseudo) for postings %d-%d:",
+	        a, b );
+	printf( "%d\t"
+	        "Leafnode placeholder for group %s\t"
+		"nobody@%s (Leafnode)\t%s\t"
+		"<leafnode.%s@%s>\t\t1000\t40\r\n",
+		b, group->name, fqdn, rfctime(), group->name, fqdn );
+	printf( ".\r\n" );
+	syslog( LOG_DEBUG, ">%d\tLeafnode placeholder for group %s\t"
+		"nobody@%s (Leafnode)\t%s\t<leafnode.%s@%s>\t\t1000\t40",
+		b, group->name, fqdn, rfctime(), group->name, fqdn );
 	return;
     }
     xovergroup = group;
@@ -781,7 +960,9 @@ void doxover(const char *arg) {
     if ( b < xfirst )
 	b = xfirst;
 
-    printf( "224 Overview information for postings %lu-%lu:\r\n", a, b );
+    printf( "224 Overview information for postings %d-%d:\r\n", a, b );
+    syslog( LOG_DEBUG, ">224 Overview information for postings %d-%d:",
+            a, b );
     for( art=a; art<=b; art++ ) {
 	if ( xoverinfo[art-xfirst].text )
 	    printf( "%s\r\n", xoverinfo[art-xfirst].text );
@@ -793,7 +974,7 @@ void doxover(const char *arg) {
 
 void dolistgroup( const char * arg ) {
     struct newsgroup * g;
-    unsigned long int art;
+    int art;
 
     if ( justaftergroup && group && group->name ) {
 	int f;
@@ -808,7 +989,8 @@ void dolistgroup( const char * arg ) {
     if ( arg && strlen(arg) ) {
 	g = findgroup( arg );
 	if ( !g ) {
-	    printf( "481 No such group: %s\r\n", arg );
+	    printf( "412 No such group: %s\r\n", arg );
+	    syslog( LOG_DEBUG, ">412 No such group: %s\r\n", arg );
 	    return;
 	} else {
 	    group = g;
@@ -818,7 +1000,8 @@ void dolistgroup( const char * arg ) {
     } else if ( group ) {
 	g = group;
     } else {
-	printf( "481 No group specified\r\n" );
+	printf( "412 No group specified\r\n" );
+	syslog( LOG_DEBUG, ">412 No group specified\r\n" );
 	return;
     }
 
@@ -827,15 +1010,20 @@ void dolistgroup( const char * arg ) {
     chdirgroup( g->name );
 
     if ( (xovergroup != group || xovergroup == NULL) && !getxover() ) {
-	printf( "481 Overview information not available\r\n" );
+	printf( "211 Article list for %s follows (pseudo)\r\n", g->name );
+	syslog( LOG_DEBUG,
+		">211 Article list for %s follows (pseudo)", g->name );
+	printf( "%d \r\n", g->last ? g->last : 1 );
+	printf( ".\r\n" );
 	return;
     }
     xovergroup = group;
 
     printf( "211 Article list for %s follows\r\n", g->name );
+    syslog( LOG_DEBUG, ">211 Article list for %s follows", g->name );
     for( art=xfirst; art<=xlast; art++ ) {
 	if ( xoverinfo[art-xfirst].text )
-	    printf( "%lu \r\n", art );
+	    printf( "%d \r\n", art );
     }
     printf( ".\r\n" );
 
