@@ -1,11 +1,21 @@
-/* $Id: texpire.c,v 1.14 1997/07/20 00:37:18 agulbra Exp $
+/*
+texpire -- expire old articles
 
 Written by Arnt Gulbrandsen <agulbra@troll.no> and copyright 1995
 Troll Tech AS, Postboks 6133 Etterstad, 0602 Oslo, Norway, fax +47
 22646949.
+Modified by Cornelius Krasel <krasel@wpxx02.toxi.uni-wuerzburg.de>
+and Randolf Skerka <rskerka@metronet.de>.
+Copyright of the modifications 1997.
+Modified by Kent Robotti <robotti@erols.com>. Copyright of the
+modifications 1998.
+Modified by Markus Enzenberger <enz@cip.physik.uni-muenchen.de>.
+Copyright of the modifications 1998.
+Modified by Cornelius Krasel <krasel@wpxx02.toxi.uni-wuerzburg.de>.
+Copyright of the modifications 1998.
 
-Use, modification and distribution is allowed without limitation,
-warranty, or liability of any kind.  */
+See file COPYING for restrictions on the use of this software.
+*/
 
 #include <ctype.h>
 #include <limits.h>
@@ -27,14 +37,12 @@ warranty, or liability of any kind.  */
 
 time_t now;
 
-struct expire_entry {
-    unsigned long xtime;
-    struct expire_entry *next;
-    char group[1];
-};
+int default_expire;
 
-struct expire_entry *expire_base = NULL;
-unsigned long default_expire;
+int verbose = 0;
+int debug = 0;
+
+struct expire_entry *expire_base;
 
 struct exp {
     char * xover;
@@ -44,7 +52,6 @@ struct exp {
 
 
 void free_expire( void ) {
-    
     struct expire_entry *a, *b;
 
     b = expire_base;
@@ -53,44 +60,13 @@ void free_expire( void ) {
         free(a);
     }
 }    
-    
-
-void readexpire( void ) {
-    FILE * f;
-    char  *l, *group, *stime, *end;
-    unsigned long i;
-    struct expire_entry *ent, *prev = NULL;
-    
-    sprintf(s, "%s/expireinfo", libdir);
-    if ((f = fopen(s, "r"))) {
-	while ((l = getaline(f))) {
-	    if ((group = strtok(l, " "))) {
-		if ((stime = strtok(NULL, "\n"))) {
-		    if ((i = strtoul(stime, &end, 0))) {
-			i = now-(time_t)(86400*i);
-			ent = (struct expire_entry *)
-			      malloc( sizeof(struct expire_entry) +
-				      strlen( group ) );
-			strcpy(ent->group, group);
-			ent->xtime = i;
-			ent->next = prev;
-			prev = ent;
-		    }
-		}
-	    }
-	    free( l );
-	}
-	fclose(f);
-	expire_base = ent;
-    }
-}    
 
 /*
 05/27/97 - T. Sweeney - Find a group in the expireinfo linked list and return
                         its expire time. Otherwise, return zero.
 */
 
-unsigned long lookup_expire(char *group) {
+int lookup_expire(char *group) {
 
     struct expire_entry *a;
    
@@ -104,131 +80,6 @@ unsigned long lookup_expire(char *group) {
 }
 
 
-/* return 1 if xover is a legal overview line, 0 else */
-
-static int legal ( char * xover ) {
-    char * p;
-    char * q;
-
-    if ( !xover )
-	return 0;
-
-    /* anything that isn't tab, printable ascii, or latin-* ? then killit */
-
-    p = xover;
-    while ( *p ) {
-	int c = (unsigned char)*p++;
-
-	if ( ( c != '\t' && c < ' ' ) || ( c > 126 && c < 160 ) )
-	    return 0;
-    }
-
-    p = xover;
-    q = strchr( p, '\t' );
-    if ( !q )
-	return 0;
-
-    /* article number */
-
-    while( p != q ) {
-	if ( !isdigit( *p ) )
-	    return 0;
-	p++;
-    }
-
-    p = q+1;
-    q = strchr( p, '\t' );
-    if ( !q )
-	return 0;
-
-    /* subject: no limitations */
-
-    p = q+1;
-    q = strchr( p, '\t' );
-    if ( !q )
-	return 0;
-
-    /* from: no limitations */
-
-    p = q+1;
-    q = strchr( p, '\t' );
-    if ( !q )
-	return 0;
-
-    /* date: no limitations */
-
-    p = q+1;
-    q = strchr( p, '\t' );
-    if ( !q )
-	return 0;
-
-    /* message-id: <*@*> */
-
-    if ( *p != '<' )
-	return 0;
-    while ( p != q && *p != '@' && *p != '>' && *p != ' ' )
-	p++;
-    if ( *p != '@' )
-	return 0;
-    while ( p != q && *p != '>' && *p != ' ' )
-	p++;
-    if ( *p != '>' )
-	return 0;
-    if ( ++p != q )
-	return 0;
-
-    p = q+1;
-    q = strchr( p, '\t' );
-    if ( !q )
-	return 0;
-
-    /* references: a series of <*@*> */
-
-    while ( p != q ) {
-	if ( *p != '<' )
-	    return 0;
-	while ( p != q && *p != '@' && *p != '>' && *p != ' ' )
-	    p++;
-	if ( *p != '@' )
-	    return 0;
-	while ( p != q && *p != '>' && *p != ' ' )
-	    p++;
-	if ( *p++ != '>' )
-	    return 0;
-	while ( p != q && *p == ' ' )
-	    p++;
-    }
-
-    p = q+1;
-    q = strchr( p, '\t' );
-    if ( !q )
-	return 0;
-
-    /* byte count */
-
-    while( p != q ) {
-	if ( !isdigit( *p ) )
-	    return 0;
-	p++;
-    }
-
-    p = q+1;
-    q = strchr( p, '\t' );
-    if ( q )
-	*q = '\0'; /* kill any extra fields */
-
-    /* line count */
-
-    while( p && *p ) {
-	if ( !isdigit( *p ) )
-	    return 0;
-	p++;
-    }
-
-    return 1;
-}
-
-
 static void dogroup ( struct newsgroup * g ) {
     char gdir[PATH_MAX];
     char *p;
@@ -236,7 +87,7 @@ static void dogroup ( struct newsgroup * g ) {
     DIR *d;
     struct dirent * de;
     struct stat st;
-    unsigned long int first, last, art;
+    int first, last, art;
     struct exp * articles;
     int n;
     int fd;
@@ -246,7 +97,7 @@ static void dogroup ( struct newsgroup * g ) {
 
     deleted = kept = 0;
     clearidtree();
-    
+ 
     /* eliminate empty groups */
 
     strcpy( gdir, spooldir );
@@ -282,10 +133,17 @@ static void dogroup ( struct newsgroup * g ) {
 		last = art;
 	}
     }
+
     if ( last < first ) {
 	closedir( d );
 	return;
     }
+    if ( verbose > 1 )
+        printf("%s: low water mark %d, high water mark %d",
+	        g->name, first, last );
+    syslog( LOG_INFO, "%s: expire %d, low water mark %d, high water mark %d",
+            g->name, expire, first, last );
+
     rewinddir( d );
 
     /* allocate and clear article array */
@@ -339,13 +197,12 @@ static void dogroup ( struct newsgroup * g ) {
 
     for ( art = first; art <= last; art++ ) {
 	if ( articles[art-first].xover &&
-	     !legal( articles[art-first].xover ) ) {	
-	    syslog( LOG_INFO, "bad overview line for %s/%lu", g->name, art );
+	     !legalxoverline( articles[art-first].xover ) ) {	
+	    syslog( LOG_INFO, "bad overview line for %s/%d", g->name, art );
 	    articles[art-first].xover = NULL;
 	}
     }
     
-
     /* insert articles in tree, and clear 'kill' for new or read articles */
 
     while ( (de=readdir(d)) ) {
@@ -407,12 +264,13 @@ static void dogroup ( struct newsgroup * g ) {
     for ( art = first; art <= last; art++ ) {
 	char name[20];
 	if ( articles[art-first].kill ) {
-	    sprintf( name, "%lu", art );
+	    sprintf( name, "%d", art );
 	    if (!unlink( name )) {
+	        syslog( LOG_DEBUG, "deleted article %s/%d", gdir, art );
 		deleted++;
 	    } else if ( errno != ENOENT ) {
 		kept++;
-		syslog( LOG_ERR, "unlink %s/%ld: %m", gdir, art );
+		syslog( LOG_ERR, "unlink %s/%d: %m", gdir, art );
 	    } else {
 		/* deleted by someone else */
 	    }
@@ -429,17 +287,24 @@ static void dogroup ( struct newsgroup * g ) {
     if ( deleted || kept )
 	printf("%s: %d articles deleted, %d kept\n", g->name, deleted, kept);
 
-    if ( !kept && !chdir("..") )
-	(void) rmdir( gdir );
+    if ( !kept ) {
+        /* reset article counters */
+        g->first = 1;
+	g->last = 0;
+	if ( unlink( ".overview" ) < 0 )
+	    syslog( LOG_ERR, "unlink %s/.overview: %m", gdir );
+	if ( !chdir("..") )
+	    (void) rmdir( gdir );
+    }
 }
     
 
 static void expiregroup( struct newsgroup * g ) {
     if ( g ) {
 	expiregroup( g->right );
-    if (!(expire = lookup_expire(g->name)))
-        expire = default_expire;
-    dogroup( g );
+	if (!(expire = lookup_expire(g->name)))
+	    expire = default_expire;
+	dogroup( g );
 	expiregroup( g->left );
     }
 }
@@ -472,7 +337,7 @@ static void expiremsgid( void ) {
 	    if ( !stat( de->d_name, &st ) )
 		if ( st.st_nlink<2 && !unlink( de->d_name ) )
 		    deleted++;
-		else
+		else if ( S_ISREG(st.st_mode) )
 		    kept++;
 	}
 	closedir( d );
@@ -485,6 +350,7 @@ static void expiremsgid( void ) {
 
 int main( int argc, char ** argv ) {
     struct passwd * pw;
+    int option;
 
     pw = getpwnam( "news" );
     if ( !pw ) {
@@ -500,7 +366,15 @@ int main( int argc, char ** argv ) {
 	exit( 1 );
     }
 
+    openlog( "texpire", LOG_PID|LOG_CONS, LOG_NEWS );
+    while ( (option=getopt( argc, argv, "v" )) != -1 ) {
+	if ( option == 'v' ) {
+            verbose++;
+	}
+    }
+
     expire = 0;
+    expire_base = NULL;
 
     readactive();
     readconfig();
@@ -509,9 +383,6 @@ int main( int argc, char ** argv ) {
 	fprintf( stderr, "%s: no expire time\n", argv[0] );
 	exit( 2 );
     }
-    free_expire();
-
-    openlog( "expire", LOG_PID|LOG_CONS, LOG_NEWS );
 
     now = time(NULL);
 
@@ -521,5 +392,3 @@ int main( int argc, char ** argv ) {
     free_expire();
     return 0;
 }
-
-	
